config_file = """# add your file containing models
models = [
    
]
# Special urlpatterns
urlpatterns = [
    'app.urls',
]

HOST = '127.0.0.1'
PORT = 5000


class BaseConfig:
    WTF_CSRF_SECRET_KEY = '{csrf}'
    SECRET_KEY = '{secret_key}'
    SQLALCHEMY_DATABASE_URI = 'sqlite:///app.db'
    SQLALCHEMY_TRACK_MODIFICATIONS = False


class ProductionConfig(BaseConfig):
    DEBUG = False
    CSRF_ENABLED = True


class DevelopConfig(BaseConfig):
    DEBUG = True
    ASSETS_DEBUG = True


Config = ProductionConfig
"""

init_file = """import sqlalchemy.ext.declarative as dec
from flask import Flask
from flask_login import LoginManager
from flask_migrate import Migrate
from flask_sqlalchemy import SQLAlchemy
from Flask_DJ.app_init import create_session
from .config import Config

app = Flask('__main__')
app.config.from_object(Config)
db = SQLAlchemy(app)
migrate = Migrate(app, db)

SqlAlchemyBase = dec.declarative_base()

login_manager = LoginManager()
login_manager.init_app(app)
session = create_session(Config.SQLALCHEMY_DATABASE_URI, SqlAlchemyBase)
"""

urls_file = """from Flask_DJ.urls import path

# Add your urls
urlpatterns = [
    
]
"""

manage_file = """from Flask_DJ import manage
from Flask_DJ.app_init import add_urls
from {project_name} import app, config

manage.init_manage(app)
manage.init_db(config.models)
for command in manage.commands:
    setattr(manage, command, manage.manager.command(getattr(manage, command)))


@manage.manager.command
def runserver():
    add_urls(config.urlpatterns)
    manage.runserver(config.HOST, config.PORT)


manage.manager.run()
"""

views_file = "# Create your views functions or classes\n"

models_file = """from {project} import db

# Create your models
"""

forms_file = """from flask_wtf import FlaskForm
import wtforms

# Create your forms
"""



from typing import List
import sqlalchemy as sa
import sqlalchemy.orm as orm
from sqlalchemy.ext.declarative import DeclarativeMeta
from sqlalchemy.orm import Session
from importlib import import_module


__factory = None


def _global_init(database_uri: str, database: DeclarativeMeta) -> None:
    global __factory

    if __factory:
        return

    conn_str = f'{database_uri}?check_same_thread=False'

    engine = sa.create_engine(conn_str, echo=False)
    __factory = orm.sessionmaker(bind=engine)

    database.metadata.create_all(engine)


def create_session(database_uri: str, database: DeclarativeMeta) -> Session:
    if not __factory:
        _global_init(database_uri, database)
    return __factory()


def add_urls(urlpatterns: List[str]):
    for file in urlpatterns:
        import_module(file)



from importlib import import_module
from warnings import warn


def add_url(app, url, func, methods):
    debug = app.debug
    if not (isinstance(url, str) and hasattr(func, '__call__')):
        warn(f"Некорректные данные", Warning)
        return False
    app.add_url_rule(url, view_func=func, methods=methods)
    print(f'add url: {url}\n' if debug else '', end='')
    return True


def path(app, url: str, func, *, methods=['GET'], _main=False):
    if not hasattr(func, '__iter__'):
        if _main:
            return add_url(app, url, func, methods)
        else:
            return url, func, methods
    else:
        for url_, fnc, methods_ in func:
            if not add_url(app, url + url_, fnc, methods_):
                return False
        return True


def include(path):
    try:
        pack = import_module(path)
    except ModuleNotFoundError:
        raise ImportError(f"Не удалось найти {path}.urlpatterns")
    try:
        patterns = getattr(pack, 'urlpatterns')
    except AttributeError:
        raise AttributeError(f'Файл {path}.urlpatterns не содержит urlpatterns')
    for args in patterns:
        if isinstance(args, bool):
            continue
        yield args



from os import makedirs, mkdir
from Flask_DJ.exceptions import CreationError
from Flask_DJ.templates import config_file, init_file, urls_file, manage_file
from string import ascii_letters
from random import choices, randint
from sys import argv


def create_file(path, name, template):
    with open(f'{path}\\{name}.py', 'w') as f:
        f.write(template)


def create_init(app_path):
    create_file(app_path, '__init__', init_file)


def generate_key(length):
    return "".join(choices(ascii_letters, k=length))


def create_config(app_path):
    create_file(app_path, 'config',
                config_file.format(csrf=generate_key(randint(9, 12)), secret_key=generate_key(randint(9, 12))))


def create_urls(app_path):
    create_file(app_path, 'urls', urls_file)


def create_main(path, name):
    app_path = f'{path}\\{name}'
    mkdir(app_path)
    create_init(app_path)
    create_config(app_path)
    create_urls(app_path)


def create_manage(path, name):
    create_file(path, 'manage', manage_file.format(project_name=name))


def create_path(name, path):
    try:
        project_path = f'{path}\\{name}' if path else name
        makedirs(project_path)
        return project_path
    except FileExistsError:
        raise CreationError('Директория с таким именем уже существует')
    except PermissionError:
        raise CreationError('У вас недостаточно прав для действий в данной директории, '
                            'попробуйте сменить пользователя и повтрить попытку')


def get_name(name):
    try:
        return name or argv[1]
    except IndexError:
        raise ValueError("Argument name is underfind")


def start(name=None, path=''):
    name = get_name(name)
    project_path = create_path(name, path)
    create_main(project_path, name)
    create_manage(project_path, name)



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='Flask-DJ',  # Required
    version='0.0.5',  # Required
    description='Django (mvc) structure for your Flask project',  # Optional
    long_description=long_description,  # Optional
    long_description_content_type='text/markdown',  # Optional (see note above)
    url='https://github.com/AlexandrovRoman/Flask-DJ',  # Optional
    author='Alexandrov Roman',  # Optional
    author_email='AlexandrovRomanisi@ya.ru',  # Optional
    classifiers=[  # Optional
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords='mvc django flask structure',  # Optional
    packages=find_packages(),  # Required
    install_requires=[
        'waitress==1.4.3',
        'flask==1.1.1',
        'Flask-Migrate==2.5.2',
        'Flask-Script==2.0.6',
        'Flask-SQLAlchemy==2.4.1',
        'WTForms==2.2.1',
        'SQLAlchemy==1.3.13',
        'Flask-WTF==0.14.3',
        'Flask-Login==0.5.0'
    ],
    entry_points={
        'console_scripts': [
            'startapp = Flask_DJ:start',
        ],
    },
)



from os import getcwd, mkdir
from os.path import exists
from flask_migrate import MigrateCommand
from flask_script import Manager
from importlib import import_module
from . import CreationError
from Flask_DJ.templates import views_file, models_file, urls_file, forms_file
from waitress import serve

"""database-methods: https://flask-migrate.readthedocs.io/en/latest/
some methods:
runserver - start server
startapp name - create folder containing forms.py, models.py, urls.py, views.py, 
"""
manager = None
app_ = None

commands = [
    'startapp',
]


def init_manage(app):
    global manager, app_
    app_ = app
    manager = Manager(app)


def init_db(models=[]):
    manager.add_command('db', MigrateCommand)
    for file in models:
        import_module(file)


def runserver(host, port):
    if app_.debug:
        app_.run(host=host, port=port)
    else:
        serve(app_, host=host, port=port)


def create_dir(name):
    try:
        mkdir(name)
    except (FileNotFoundError, PermissionError):
        raise CreationError('Не удается создать директорию с данным именем')


def create_files(name):
    with open(f'{name}/views.py', 'w') as f:
        f.write(views_file)
    with open(f'{name}/models.py', 'w') as f:
        f.write(models_file.format(project=getcwd().split('\\')[-1]))
    with open(f'{name}/urls.py', 'w') as f:
        f.write(urls_file)
    with open(f'{name}/forms.py', 'w') as f:
        f.write(forms_file)


def warning_handler(message):
    commands = {1: lambda: None, 2: exit}
    try:
        commands[int(input(f'{message}\n1. Yes\n2. No'))]()
    except (ValueError, KeyError):
        print('Incorrect input')
        exit(1)


def startapp(name):
    if exists(name):
        warning_handler('Директория с данным именем уже существует, все равно создать?')
    else:
        create_dir(name)
    create_files(name)
    print(f'app {name} created')
